package cn.ninegame.genericframework.basic;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.SparseArray;

import java.util.Iterator;
import java.util.Stack;

import cn.ninegame.genericframework.ui.BaseActivity;
import cn.ninegame.genericframework.ui.BaseFragment;
import cn.ninegame.genericframework.ui.FragmentCenter;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-03-18
 * Author      : lihq@ucweb.com
 * Description : 封装了发送消息的接口，屏蔽MsgBroker不可见。作用类似于Context上下文。
 * ****************************************************************************
 */
final class EnvironmentImpl implements Environment {

    MsgBroker mMsgBroker;
    FragmentCenter mFragmentCenter;
    NotificationCenter mNotificationCenter;
    Context mApplicationContext;
    Activity mCurrentActivity;
    EnvironmentInterceptor mInterceptor;

    @Override
    public int getMessageIdByName(String messageName) {
        return mMsgBroker.getMessageIdByName(messageName);
    }

    /**
     * Activity栈
     */
    SparseArray<Stack<Activity>> mTaskArray = new SparseArray<Stack<Activity>>();
    /**
     * TaskId栈
     * */
    Stack<Integer> mTaskIdStack = new Stack<Integer>();

    @Override
    public void sendMessage(String messageName) {
        sendMessage(getMessageIdByName(messageName));
    }

    @Override
    public void sendMessage(String messageName, Bundle messageData) {
        sendMessage(getMessageIdByName(messageName),messageData);
    }

    @Override
    public void sendMessageForResult(String messageName, Bundle messageData, IResultListener listener) {
        sendMessageForResult(getMessageIdByName(messageName),messageData,listener);
    }

    @Override
    public Bundle sendMessageSync(String messageName) {
        return sendMessageSync(getMessageIdByName(messageName));
    }

    @Override
    public Bundle sendMessageSync(String messageName, Bundle messageData) {
        return sendMessageSync(getMessageIdByName(messageName),messageData);
    }

    @Override
    public void sendMessage(int messageId) {
        mMsgBroker.sendMessage(messageId, null);
    }

    @Override
    public void sendMessage(int messageId, Bundle messageData) {
        mMsgBroker.sendMessage(messageId, messageData);
    }

    @Override
    public void sendMessageForResult(int messageId, Bundle messageData, IResultListener listener) {
        mMsgBroker.sendMessageForResult(messageId, messageData, listener);
    }

    @Override
    public Bundle sendMessageSync(int messageId) {
        return mMsgBroker.sendMessageSync(messageId);
    }

    @Override
    public void sendMessage(String messageName, IMessageContext messageContext) {
        sendMessage(getMessageIdByName(messageName),messageContext);
    }

    @Override
    public void sendMessage(int messageId, IMessageContext messageContext) {
        mMsgBroker.sendMessageForResult(messageId, null, null,messageContext);
    }

    @Override
    public Bundle sendMessageSync(int messageId, IMessageContext messageContext) {
        return mMsgBroker.sendMessageSync(messageId,null,messageContext);
    }

    @Override
    public void sendMessage(String messageName, IMessageContext messageContext, Bundle messageData) {
        sendMessage(getMessageIdByName(messageName),messageContext,messageData);
    }

    @Override
    public void sendMessageForResult(String messageName, IMessageContext messageContext, Bundle messageData, IResultListener listener) {
        sendMessageForResult(getMessageIdByName(messageName),messageContext,messageData,listener);
    }

    @Override
    public Bundle sendMessageSync(String messageName, IMessageContext messageContext) {
        return sendMessageSync(getMessageIdByName(messageName),messageContext);
    }

    @Override
    public void sendMessage(int messageId, IMessageContext messageContext, Bundle messageData) {
        mMsgBroker.sendMessageForResult(messageId,messageData,null,messageContext);
    }

    @Override
    public void sendMessageForResult(int messageId, IMessageContext messageContext, Bundle messageData, IResultListener listener) {
        mMsgBroker.sendMessageForResult(messageId,messageData,listener,messageContext);
    }

    @Override
    public Bundle sendMessageSync(int messageId, IMessageContext messageContext, Bundle messageData) {
        return  mMsgBroker.sendMessageSync(messageId,messageData,messageContext);
    }

    @Override
    public Bundle sendMessageSync(String messageName, IMessageContext messageContext, Bundle messageData) {
        return sendMessageSync(getMessageIdByName(messageName),messageContext,messageData);
    }

    @Override
    public Bundle sendMessageSync(int messageId, Bundle messageData) {
        return mMsgBroker.sendMessageSync(messageId,messageData);
    }

    public void setMsgBroker(MsgBroker msgBroker) {
        mMsgBroker = msgBroker;
    }

    public void setNotificationCenter(NotificationCenter center){
        mNotificationCenter = center;
    }

    private void pushToTaskIdStack(int taskId) {
        Integer current = taskId;
        if (mTaskIdStack.size() == 0) {
            //空，直接push
            mTaskIdStack.push(current);
        } else {
            Integer top = mTaskIdStack.peek();
            //已经是栈顶
            if (top.intValue() == taskId) {
                return;
            } else {
                //不在栈顶，先删除
                removeToTaskIdStack(taskId);
                //再push
                mTaskIdStack.push(current);
            }
        }
    }


    private void removeToTaskIdStack(int taskId) {
        Iterator<Integer> iterator = mTaskIdStack.iterator();
        while (iterator.hasNext()) {
            Integer act = iterator.next();
            if (act.intValue() == taskId) {
                iterator.remove();
                break;
            }
        }
    }



    @Override
    public void sendNotification(Notification notification) {
        mNotificationCenter.sendNotification(notification);
    }

    public void registerNotification(int notificationId, INotify notify){
        mNotificationCenter.register(notificationId, notify);
    }

    public void unregisterNotification(int notificationId, INotify notify){
        mNotificationCenter.unregister(notificationId, notify);
    }

    @Override
    public void sendNotification(int notificationId, Bundle bundleData) {
        mNotificationCenter.sendNotification(notificationId,bundleData);
    }

    @Override
    public void sendNotification(String notificationName, Bundle bundleData) {
        mNotificationCenter.sendNotification(notificationName,bundleData);
    }

    @Override
    public void registerNotification(String notificationName, INotify notify) {
        mNotificationCenter.register(notificationName, notify);
    }

    @Override
    public void registerNotificationWithId(int notificationId, INotify notify) {
        mNotificationCenter.register(notificationId, notify);
    }

    @Override
    public void unregisterNotification(String notificationName, INotify notify) {
        mNotificationCenter.unregister(notificationName,notify);
    }

    @Override
    public void unregisterNotificationWithId(int notificationId, INotify notify) {
        mNotificationCenter.unregister(notificationId, notify);
    }

    void setInterceptor(EnvironmentInterceptor interceptor) {
        mInterceptor = interceptor;
    }

    public void setFragmentCenter(FragmentCenter center) {
        mFragmentCenter = center;
    }



    public synchronized void pushToActivityStackIfNeed(Activity activity) {
        if (mTaskArray.size() > 0) {
            //remove Duplicate Activity
            removeActivity(activity);
        }
        pushToActivityStack(activity);
    }

    /**
     * Activity进栈
     * @param activity
     */
    public synchronized void pushToActivityStack(Activity activity){
        int taskId = activity.getTaskId();
        if(mTaskArray.get(taskId) == null){
            mTaskArray.put(taskId, new Stack<Activity>());
        }
        Stack<Activity> stack = mTaskArray.get(taskId);
        stack.push(activity);
        pushToTaskIdStack(taskId);

        mCurrentActivity = activity;
    }

    /**
     * 弹出栈顶Activity类对象
     * 这个函数暂时没用
     */
    public synchronized Activity popActivityStack() {
        if (mTaskArray.size() == 0) {
            return null;
        }
        removeActivity(mCurrentActivity);
        return mCurrentActivity;
    }

    public synchronized void removeActivity(Activity activity){
        if (mTaskArray.size() == 0) {
            return ;
        }

        int taskId = activity.getTaskId();
        Stack<Activity> stack = mTaskArray.get(taskId);
        if(stack != null && stack.size() > 0){
            stack.remove(activity);
            if (stack.size() == 0) {
                mTaskArray.remove(taskId);
                removeToTaskIdStack(taskId);
            }
        }
    }

    /**
     * 清空栈内所有的activity
     */
    public synchronized void finishAllActivities(){
        try {
            int key;
            Stack<Activity> stack;
            for (int i = 0; i < mTaskArray.size(); i++) {
                key = mTaskArray.keyAt(i);
                stack = mTaskArray.get(key);
                if (stack != null) {
                    Iterator<Activity> iterator = stack.iterator();
                    while (iterator.hasNext()) {
                        Activity act = iterator.next();
                        iterator.remove();
                        if (act != null) {
                            act.finish();
                        }
                    }
                }
                removeToTaskIdStack(key);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 返回Activity栈的大小
     * @return
     */
    public synchronized int getActivityStackSize(){
        int key = 0;
        int stackSize = 0;
        Stack<Activity> stack;
        for (int i = 0; i < mTaskArray.size(); i++) {
            key = mTaskArray.keyAt(i);
            stack = mTaskArray.get(key);
            if (stack != null) {
                stackSize += stack.size();
            }
        }
        return stackSize;
    }

    /**
     * 获取当前Activity栈
     * @return
     */
    public synchronized SparseArray<Stack<Activity>> getActivityStack(){
        return mTaskArray;
    }

    @Override
    public Context getApplicationContext() {
        return mApplicationContext;
    }

    @Override
    public void setApplicationContext(Context applicationContext) {
        mApplicationContext = applicationContext;
    }

    @Override
    public synchronized Activity getCurrentActivity() {
        Activity activity = null;
        if (getActivityStackSize() == 0) {
            activity = mCurrentActivity;
        } else {
            Integer taskid = mTaskIdStack.peek();
            Stack<Activity> stack = mTaskArray.get(taskid);
            if (stack != null) {
                activity = stack.peek();
            }
        }
        return activity;
    }

    @Override
    public void startFragment(String fragmentID) {
        startFragment(fragmentID,null);
    }

    @Override
    public void startFragment(String fragmentID, Bundle param){
        startFragment(fragmentID, param, FrameworkFacade.getInstance().getConfig().isOpenGlobalAnim(), BaseFragment.LAUNCHER_MODE_NORMAL);
    }


    @Override
    public void startFragmentForResult(String fragmentID, Bundle param , IResultListener listener) {
        startFragmentForResult(fragmentID,param,listener,FrameworkFacade.getInstance().getConfig().isOpenGlobalAnim(),BaseFragment.LAUNCHER_MODE_NORMAL);
    }

    @Override
    public void startFragment(String fragmentID, Bundle param, boolean useAnim, int mode){
        startFragmentForResult(fragmentID, param, null, useAnim, mode);
    }

    @Override
    public void startFragmentForResult(String fragmentID, Bundle param, IResultListener listener, boolean useAnim, int mode) {
        if (mInterceptor.needInterceptFragment(fragmentID,param)) {
            mInterceptor.interceptFragment(fragmentID,param,listener,useAnim,mode);
        }else {
            startFragmentForResultNoIntercept(fragmentID,param,listener,useAnim,mode);
        }
    }

    @Override
    public void startFragmentForResultNoIntercept(String fragmentID, Bundle param, IResultListener listener, boolean useAnim, int mode) {
        saveSourceListener(listener);
        mFragmentCenter.startFragmentForResult(fragmentID, param, listener, useAnim, mode);
    }

    private void saveSourceListener(IResultListener listener) {
        if (getCurrentActivity() instanceof BaseActivity) {
            Fragment fragment = ((BaseActivity) getCurrentActivity()).getCurrentFragment();
            if (fragment != null && fragment instanceof BaseFragment) {
                ((BaseFragment) fragment).setSourceResultListener(listener);
            }
        }
    }

    @Override
    public void startFragments(String[] fragmentIDs, Bundle[] params){
        if (fragmentIDs != null && fragmentIDs.length > 0){
            int [] modes = new int[fragmentIDs.length];
            for (int i=0; i<modes.length; ++i){
                modes[i] = BaseFragment.LAUNCHER_MODE_NORMAL;
            }
            startFragments(fragmentIDs, params, modes);
        }else{
            new Throwable("fragmentIDs can not be null or length less than 0 !");
        }
    }

    @Override
    public void startFragments(String[] fragmentIDs, Bundle[] params, int[] modes) {
        if (fragmentIDs != null && fragmentIDs.length > 0) {
            boolean bNeedIntercept = false;
            for (int i = 0;i<fragmentIDs.length;i++) {
                bNeedIntercept = mInterceptor.needInterceptFragment(fragmentIDs[i],params[i]);
                //只要有一个fragment需要拦截，那么整个startFragments都会被拦截。
                if (bNeedIntercept) {
                    break;
                }
            }

            if (bNeedIntercept) {
                mInterceptor.interceptMutilFragment(fragmentIDs,params,modes);
            }else {
                startFragmentsNoIntercept(fragmentIDs,params,modes);
            }
        }
    }

    @Override
    public void startFragmentsNoIntercept(String[] fragmentIDs, Bundle[] params, int[] modes) {
        mFragmentCenter.startFragments(fragmentIDs, params, modes);
    }

    public void startDialogFragment(String fragmentID, Bundle param){
        startDialogFragment(fragmentID, param, null);
    }

    public void startDialogFragment(String fragmentID, Bundle param, IResultListener listener){
        if (mInterceptor.needInterceptFragment(fragmentID,param)) {
            mInterceptor.interceptDialogFragment(fragmentID,param,listener);
        }else {
            startDialogFragmentNoIntercept(fragmentID, param, listener);
        }
    }

    @Override
    public void startDialogFragmentNoIntercept(String fragmentID, Bundle param, IResultListener listener) {
        mFragmentCenter.startDialogFragment(fragmentID, param, listener);
    }

    @Override
    public void startFragmentWithFlag(String fragmentID, Bundle param, boolean useAnim, int intentFlag){
        if (mInterceptor.needInterceptFragment(fragmentID,param)) {
            mInterceptor.interceptFragmentWithFlags(fragmentID, param, useAnim, intentFlag);
        }else {
            startFragmentWithFlagNoIntercept(fragmentID, param, useAnim, intentFlag);
        }
    }

    @Override
    public void startFragmentWithFlagNoIntercept(String fragmentName, Bundle param, boolean useAnim, int intentFlag) {
        mFragmentCenter.startFragmentWithFlag(fragmentName, param, useAnim, intentFlag);
    }

    @Override
    public void reInitFragment(Fragment fragment) {
        mFragmentCenter.reInitFragment(fragment);
    }

    @Override
    public void loadFragment(String fragmentID, ILoadFragmentListener listener) {
        mFragmentCenter.loadFragment(fragmentID,listener);
    }

    @Override
    public void startFragmentWithPop(String fragmentName, Bundle param) {
        startFragmentWithPop(fragmentName, param, FrameworkFacade.getInstance().getConfig().isOpenGlobalAnim());
    }

    @Override
    public void startFragmentWithPop(String fragmentName, Bundle param, boolean useAnim) {
        startFragment(fragmentName, param, useAnim, BaseFragment.LAUNCHER_MODE_START_WITH_POP);
    }

    @Override
    public void startFragmentWithHide(String fragmentName, Bundle param) {
        startFragmentWithHide(fragmentName, param, FrameworkFacade.getInstance().getConfig().isOpenGlobalAnim());
    }

    @Override
    public void startFragmentWithHide(String fragmentName, Bundle param, boolean useAnim) {
        startFragment(fragmentName, param, useAnim, BaseFragment.LAUNCHER_MODE_START_WITH_HIDE);
    }

}
