package cn.ninegame.genericframework.basic;

import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;

import com.aligame.framework.BuildConfig;

import java.lang.ref.WeakReference;
import java.lang.reflect.Method;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import cn.ninegame.genericframework.module.IModuleManifest;
import cn.ninegame.genericframework.tools.CRC16;

/**
 * Changed      : zhonghui.jzh
 * Description : 通知中心
 * ****************************************************************************
 */
class NotificationCenter {
    private static final String TAG = "NotificationCenter";
    private Map<Integer,ArrayList<WeakReference<Object>>> mNotifyMap = new ConcurrentHashMap<Integer, ArrayList<WeakReference<Object>>>();
    private Map<Integer,GundamNotification> mNotiyEntryMap = new ConcurrentHashMap<Integer, GundamNotification>();
    private Map<String,Integer> mIdAndNamesEntryMap = new ConcurrentHashMap<String, Integer>();

    private static final int NOTIFICATION_MESSAGE_ID = 2;

    public Handler mHandler;
    private ExecutorService mBackgroundExecutor = null;

    public NotificationCenter() {

    }

    public void init(IModuleManifest[] moduleManifests){
        if(moduleManifests!=null){
            for(IModuleManifest moduleManifest: moduleManifests){
                if(moduleManifest.getNotifications()!=null){
                    for(GundamNotification gundamNotification:moduleManifest.getNotifications()){
                        mNotiyEntryMap.put(gundamNotification.id,gundamNotification);
                        mIdAndNamesEntryMap.put(gundamNotification.name,gundamNotification.id);
                    }
                }
            }
        }

        mHandler = new Handler(Looper.getMainLooper(),new Handler.Callback() {
            @Override
            public boolean handleMessage(Message message) {
                if (message.what == NOTIFICATION_MESSAGE_ID) {
                    notifyInner((Notification) message.obj);
                    return true;
                }
                return false;
            }
        });

        if(mBackgroundExecutor==null){
            int cpuCount = Runtime.getRuntime().availableProcessors();
            mBackgroundExecutor = Executors.newFixedThreadPool(cpuCount + 1);
        }
    }

    public void register(int notificationID, Object notify) {
        if (notificationID ==0 || notify == null) {
            return;
        }
        ArrayList<WeakReference<Object>> arrayList = mNotifyMap.get(notificationID);

        if (arrayList == null) {
            arrayList = new ArrayList<WeakReference<Object>>();
            mNotifyMap.put(notificationID, arrayList);
        }

        arrayList.add(new WeakReference<Object>(notify));
    }

    public void register(String notificationName, Object notify) {
        if (TextUtils.isEmpty(notificationName) || notify == null) {
            return;
        }
        if(mIdAndNamesEntryMap.containsKey(notificationName)){
            int notificationID = mIdAndNamesEntryMap.get(notificationName);
            ArrayList<WeakReference<Object>> arrayList = mNotifyMap.get(notificationID);
            if (arrayList == null) {
                arrayList = new ArrayList<WeakReference<Object>>();
                mNotifyMap.put(notificationID, arrayList);
            }
            arrayList.add(new WeakReference<Object>(notify));
        }else{
            int notificationID = crcNotificationIdCode(notificationName);
            GundamNotification gundamNotification = new GundamNotification();
            gundamNotification.id = notificationID;
            gundamNotification.name = notificationName;
            gundamNotification.threadMode = "posting";
            gundamNotification.method = "";

            mNotiyEntryMap.put(notificationID,gundamNotification);
            mIdAndNamesEntryMap.put(notificationName,gundamNotification.id);

            ArrayList<WeakReference<Object>> arrayList = mNotifyMap.get(notificationID);
            if (arrayList == null) {
                arrayList = new ArrayList<WeakReference<Object>>();
                mNotifyMap.put(notificationID, arrayList);
            }
            arrayList.add(new WeakReference<Object>(notify));

        }
    }


    public int crcNotificationIdCode(String msg) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("_");
        stringBuffer.append(msg);
        stringBuffer.append("_");
        int crc16 = CRC16.calcCrc16(stringBuffer.toString().getBytes());
        //00 default,01 notification type,
        return new BigInteger(String.format("0001%04x",crc16), 16).intValue();
        //return stringBuffer.toString().hashCode();
    }

    /**
     * use in debug/develop mode
     * @param notificationID
     * @param notify
     * @return
     */
    private boolean isDuplicateRegister( int notificationID, INotify notify) {
        ArrayList<WeakReference<Object>> notifyList = mNotifyMap.get(notificationID);

        if(notifyList != null && notifyList.size() > 0){
            Object arrayMember;

            for (WeakReference<Object> weakObject : notifyList){
                if(weakObject == null){
                    continue;
                }

                arrayMember = weakObject.get();
                if (arrayMember != null && arrayMember == notify) {
                    return true;
                }
            }
        }

        return false;
    }

    public void unregister(int notificationID, Object notify){

        ArrayList<WeakReference<Object>> refList = mNotifyMap.get(notificationID);

        if(refList != null) {
            int size = refList.size();
            for (int i = 0; i < size; i++) {
                WeakReference<Object> ref = refList.get(i);
                if (ref == null) {
                    continue;
                }
                Object refNotify = ref.get();
                if (refNotify != null && refNotify == notify) {
                    refList.remove(ref);
                    break;
                }
            }
        }
    }

    public void unregister(String notificationName, Object notify){
        if (TextUtils.isEmpty(notificationName) || notify == null) {
            return;
        }
        if(mIdAndNamesEntryMap.containsKey(notificationName)) {
            int notificationID = mIdAndNamesEntryMap.get(notificationName);
            ArrayList<WeakReference<Object>> refList = mNotifyMap.get(notificationID);

            if (refList != null) {
                int size = refList.size();
                for (int i = 0; i < size; i++) {
                    WeakReference<Object> ref = refList.get(i);
                    if (ref == null) {
                        continue;
                    }

                    Object refNotify = ref.get();
                    if (refNotify != null && refNotify == notify) {
                        refList.remove(ref);
                        break;
                    }
                }
            }
        }
    }

    public void sendNotification(Notification notification){
        if(notification.id>0){
            sendNotification(notification.id,notification.bundleData);
        }else if(!TextUtils.isEmpty(notification.messageName)){
            sendNotification(notification.messageName,notification.bundleData);
        }
    }

    public void sendNotification(final String notificationName, final Bundle bundleData){
        Integer notificationId = mIdAndNamesEntryMap.get(notificationName);
        if(notificationId!=null){
            sendNotification(notificationId.intValue(),bundleData);
        }else{
            String msg = "notifyInner can not find event GundamNotification Id notification name:"+ notificationName;
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
        }
    }

    public void sendNotification(final int notificationId, final Bundle bundleData){
        GundamNotification gundamNotification = mNotiyEntryMap.get(notificationId);
        if(gundamNotification==null){
            statNotificationNotFindException(notificationId);
            String msg = "notifyInner can not find event GundamNotification for notification,id:"+ getLogNotificationId(notificationId);
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
            return;
        };
        if(TextUtils.isEmpty(gundamNotification.threadMode)){
            gundamNotification.threadMode="posting";
        }
        switch (gundamNotification.threadMode){
            case "main":
                if(isMainThread()){
                    notifyInner(new Notification(notificationId,bundleData));
                }else{
                    mHandler.sendMessage(mHandler.obtainMessage(NOTIFICATION_MESSAGE_ID, new Notification(notificationId,bundleData)));
                }
                break;
            case "async":
            case "posting":
                notifyInner(new Notification(notificationId,bundleData));
                break;
            case "background":
                mBackgroundExecutor.execute(new Runnable() {
                    @Override
                    public void run() {
                        notifyInner(new Notification(notificationId,bundleData));
                    }
                });
                break;
        }
    }

    public void sendNotification(int notificationId, Bundle bundleData,long delay){
        if(isMainThread()){
            notifyInner(new Notification(notificationId,bundleData));
        }else{
            mHandler.sendMessageDelayed(mHandler.obtainMessage(NOTIFICATION_MESSAGE_ID, new Notification(notificationId,bundleData)), delay);
        }
    }

    public void sendNotification(Notification notification,long delay){
        mHandler.sendMessageDelayed(mHandler.obtainMessage(NOTIFICATION_MESSAGE_ID, notification), delay);
    }

    private void notifyInner(Notification notification){
        GundamNotification gundamNotification = mNotiyEntryMap.get(notification.id);
        if(gundamNotification==null){
            statNotificationNotFindException(notification.id);
            String msg = "notifyInner can not find event GundamNotification for notification,id:"+ getLogNotificationId(notification.id) + " execute thread:" + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
            return;
        };
        ArrayList<WeakReference<Object>> weakRefList = mNotifyMap.get(notification.id);
        if (weakRefList != null ){
            int size = weakRefList.size();
            try {
                for (int i = 0; i < size; i++) {
                    WeakReference<Object> weakNotify = weakRefList.get(i);
                    if (weakNotify != null) {
                        Object notify = weakNotify.get();
                        if (notify != null) {
                            long startTime = SystemClock.elapsedRealtime();
                            try {
                                Class ownerClass = notify.getClass();
                                Class[] argsClass = new Class[]{Notification.class};
                                Notification notificationMessage = new Notification(notification.id,gundamNotification.name,notification.bundleData);
                                if(TextUtils.isEmpty(gundamNotification.method)){
                                    //notify.onNotify(notification);
                                    gundamNotification.method ="onNotify";
                                    Method method = ownerClass.getDeclaredMethod("onNotify",argsClass);
                                    method.setAccessible(true);
                                    method.invoke(notify, new Object[]{notificationMessage});
                                }else{
                                    Method method = ownerClass.getDeclaredMethod(gundamNotification.method,argsClass);
                                    method.setAccessible(true);
                                    method.invoke(notify, new Object[]{notificationMessage});
                                }
                                if(FrameworkService.getInstance().mIsDebug){
                                    String logMessage = "handleNotification success to dispatcher notification "+notify.getClass().getSimpleName()+",id:"+ getLogNotificationId(notification.id)+",notification name:" + gundamNotification.name+ ",target:" + gundamNotification.method+ " on thread " + gundamNotification.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread");
                                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,logMessage);
                                }
                            }catch (Exception ne){
                                ne.printStackTrace();
                                Log.e(TAG, ne.toString());
                                statNotificationException(notification,notify.getClass().getSimpleName());
                                String messageLog = "handleNotification execute exception when dispatcher notification "+notify.getClass().getSimpleName()+",id:"+getLogNotificationId(notification.id)+",notification name:" + gundamNotification.name+ ",target:" + gundamNotification.method+ " on thread " + gundamNotification.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread");
                                FrameworkService.addDebugLog(TAG, messageLog, ne);
                            }
                            long timeInterval = SystemClock.elapsedRealtime() - startTime;
                            if (timeInterval > StatManager.ANR_INTERVAL){
                                HashMap<String, String> data = new HashMap<String, String>();
                                data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_NOTIFICATION_JANK);
                                data.put(StatManager.Constant.KEY_NOTIFICATION_ID, String.valueOf(gundamNotification.id));
                                data.put(StatManager.Constant.KEY_NOTIFICATION_NAME, gundamNotification.name);
                                data.put(StatManager.Constant.KEY_NOTIFICATION_THREAD_MODE, gundamNotification.threadMode);
                                data.put(StatManager.Constant.KEY_NOTIFICATION_CLASS, notify.getClass().getSimpleName());
                                data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(timeInterval));
                                FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,"sendMessageSync处理超过2秒 " + data.toString());
                                FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
                            }
                        }
                    }
                }
            }catch (Exception e){
                Log.e(TAG, e.toString());
                e.printStackTrace();
                statNotificationException(notification,"");
                if(TextUtils.isEmpty(gundamNotification.method)){
                    gundamNotification.method = "onNotify";
                }
                String messageLog = "handleNotification execute exception when dispatcher notification,id:"+getLogNotificationId(notification.id)+",notification name:" + gundamNotification.name+ ",target:" + gundamNotification.method+ " on thread " + gundamNotification.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread");
                FrameworkService.addDebugLog(TAG, messageLog, e);
            }
        }else{
            String msg = "notifyInner can not find Notification handler for notification,id:"+ getLogNotificationId(notification.id) + " execute thread:" + (isMainThread()?"main thread":"background thread");
            statNotificationException(notification,"");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
        }
        //考虑注册清理
    }


    private void statNotificationException(Notification notification,String handlerName){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_NOTIFICATION_EXP);
        data.put(StatManager.Constant.KEY_NOTIFICATION_ID, getLogNotificationId(notification.id));
        data.put(StatManager.Constant.KEY_NOTIFICATION_NAME, notification.messageName);
        data.put(StatManager.Constant.KEY_NOTIFICATION_CLASS, handlerName);
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }

    private void statNotificationNotFindException(int messageId){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_NOTIFICATION_DEF_EXP);
        data.put(StatManager.Constant.KEY_NOTIFICATION_ID, getLogNotificationId(messageId));
        data.put(StatManager.Constant.KEY_NOTIFICATION_NAME, "");
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }
    private void statNotificationNotFindException(String messageName){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_NOTIFICATION_DEF_EXP);
        data.put(StatManager.Constant.KEY_NOTIFICATION_ID, "");
        data.put(StatManager.Constant.KEY_NOTIFICATION_NAME, messageName);
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }


    private boolean isMainThread(){
        return Thread.currentThread().getId() == Looper.getMainLooper().getThread().getId();
    }

    private String getLogNotificationId(int notificationId){
        return String.valueOf(notificationId) + "(" + String.format("0x%08x", notificationId) +")";
    }

}
