/*
 *****************************************************************************
 * Copyright (C) 2005-2015 UCWEB Inc. All rights reserved
 * Description :
 * Creation    : 2015/04/21
 * Author      : peidong@ucweb.com
 * History     :
 *               Creation, 2015/04/21, peidong, Create the file
 ******************************************************************************
 **/
package cn.ninegame.genericframework.tools.dalvik;


import java.util.HashMap;

import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.StatManager;
import cn.ninegame.genericframework.tools.LOG;

public class DalvikPatch {
    private final static String TAG = "DalvikPatch";
    private static boolean sInit = true;

    static {
        try{
            System.loadLibrary("dalvikpatch");
        }catch (Throwable e){
            sInit = false;
            e.printStackTrace();
        }
    }

    public static void patchIfPossible() {
        if (sInit == false) {
            return;
        }
        long start = System.currentTimeMillis();
        boolean isDalvik = isDalvik();
        int beforeMapLen = getMapLength();
        int beforeMapUsed = getMapUsed();

        LOG.d(TAG, "isDalvik:" + isDalvik);
        LOG.d(TAG, "before adjustLinearAlloc:");
        LOG.d(TAG, "getMapLength():" + beforeMapLen );
        LOG.d(TAG, "getMapUsed():" + beforeMapUsed);

        int result = adjustLinearAlloc();
        LOG.d(TAG, "adjustLinearAlloc result: " + result);
        LOG.d(TAG, "adjustLinearAlloc error: " + getError());

        LOG.d(TAG, "after adjustLinearAlloc:");
        LOG.d(TAG, "getMapLength():" + getMapLength());
        LOG.d(TAG, "getMapUsed():" + getMapUsed());

        long end = System.currentTimeMillis() - start;
        LOG.d(TAG, "adjust linearAlloc take time:" + end);

        statLinearAlloc(isDalvik, beforeMapLen, beforeMapUsed,result, getError(), getMapLength(), getMapUsed(), end);
    }

    public static void statLinearAlloc(boolean isDalvik, int beforeLenght, int beforeUsedLenght, int result, int error, int afterLength, int afterUsedLength, long time){

        HashMap<String, String> data = new HashMap<String, String>();

        data.put(StatManager.Constant.KEY_LINEAR_ALLOC, StatManager.Constant.VALUE_LINEAR_ALLOC);

        data.put(StatManager.Constant.KEY_IS_DVM, String.valueOf(isDalvik));

        data.put(StatManager.Constant.KEY_BEFORE_AJUST_LINEAR_ALLOC_LENGTH, isDalvik ? String.valueOf(beforeLenght) : "0");
        data.put(StatManager.Constant.KEY_BEFORE_AJUST_LINEAR_ALLOC_USED_LENGTH, isDalvik ? String.valueOf(beforeUsedLenght) : "0");
        data.put(StatManager.Constant.KEY_LINEAR_ALLOC_RESULT, isDalvik ? String.valueOf(result) : "-1");
        data.put(StatManager.Constant.KEY_LINEAR_ALLOC_ERROR, isDalvik ?  String.valueOf(error) : "-1");

        data.put(StatManager.Constant.KEY_AFTER_AJUST_LINEAR_ALLOC_LENGTH, isDalvik ? String.valueOf(afterLength) : "0");
        data.put(StatManager.Constant.KEY_AFTER_AJUST_LINEAR_ALLOC_USED_LENGTH, isDalvik ? String.valueOf(afterUsedLength) : "0");
        data.put(StatManager.Constant.KEY_LINEAR_ALLOC_TIME, String.valueOf(time));

        FrameworkService.getInstance().getService(StatManager.class).addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
    }

    private static native boolean isDalvik();

    private static native int getError();

    private static native int getMapUsed();

    private static native int getMapLength();

    private static native int adjustLinearAlloc();
}
