package cn.ninegame.genericframework.ui;

import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import cn.ninegame.genericframework.tools.ViewIdGenerator;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 带有ViewPagper的Fragment
 * <p>Copyright: Copyright (c) 2015</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2015/12/2
 */
public abstract class BaseViewPagerFragment extends BaseFragment {
    protected ViewPager mViewPager;
    private static final int DEFAULT_PAGE_LIMIT = 1;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        if (mViewPager == null){
            initViewPager(inflater);
        }else{
            ViewGroup parent = (ViewGroup) mViewPager.getParent();
            if (parent != null) {
                parent.removeView(mViewPager);
            }
        }

        return mViewPager;
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        view.setClickable(true);
    }

    private View initViewPager(LayoutInflater inflater){
        mViewPager = createViewPager(inflater);
        mViewPager.setAdapter(new FragmentStatePagerAdapter(getChildFragmentManager()) {
            @Override
            public Fragment getItem(final int position) {
                BaseFragment fragment = BaseViewPagerFragment.this.getItem(position);
                fragment.setEnvironment(getEnvironment());
                return fragment;
            }

            @Override
            public int getCount() {
                return BaseViewPagerFragment.this.getCount();
            }

            @Override
            public CharSequence getPageTitle(int position) {
                return BaseViewPagerFragment.this.getPageTitle(position);
            }
        });

        return mViewPager;
    }

    /**
     * 子类可重写该方法，使用自定义的ViewPager
     * @param inflater
     * @return
     */
    public ViewPager createViewPager(LayoutInflater inflater){
        ViewPager vp = new ViewPager(inflater.getContext());
        vp.setId(ViewIdGenerator.generateViewId());
        return vp;
    }

    /**
     * 滑动过程中时候是否一直持有Fragment对象不释放
     * @param isKeep
     */
    public void setAlwaysKeepPager(boolean isKeep){
        if (isKeep){
            mViewPager.setOffscreenPageLimit(getCount());
        }else {
            mViewPager.setOffscreenPageLimit(DEFAULT_PAGE_LIMIT);
        }
    }

    public abstract BaseFragment getItem(int position);

    /**
     * 获取page页面个数
     * @return
     */
    public abstract int getCount();

    public abstract CharSequence getPageTitle(int position);
}
